// ----------------------------------------------------------------------------
//	Copyright (c) 2001-2002 Thomas E. Hilinski. All rights reserved.
//
//	This software is made available under the terms of the
//	GNU Lesser General Public License, found at:
//	http://www.gnu.org/licenses/licenses.html#LGPL
//	This copyright notice must remain intact and in place.
// ----------------------------------------------------------------------------
//	Project:  char and char* utility functions
//	File:	  charutil.cpp
//
//	Description: See header file.
// ----------------------------------------------------------------------------
//	Author:	Tom Hilinski, hilinski@lamar.colostate.edu, Aug2002
//	History: See header file.
// ----------------------------------------------------------------------------
//	This version added to VIDE source package for VIDE version 2.00
//	by Bruce Wampler date: 
// ----------------------------------------------------------------------------

#include "charutil.h"
#include <cctype>
using namespace std;

//	-----------------------------------------------------------------------
//	strtrim
//	Trims the leading and trailing spaces from a string.
//	Returns a pointer to the string.
char* strtrim (char* s)
{
	if ( !s || !(*s) )				// anything there?
		return s;

	register char* p = s + strlen(s) - 1;	// point to end

	// trim trailing space
	while ( p >= s && (*p < '!' || *p > '~') )
		--p;
	*(++p) = '\0';			// terminate with NULL

	// trim leading space (if any)
	if ( *s < '!' || *s > '~' )
	{
		char *pEnd = p;			// point to terminating NULL
		p = s + 1;			// start of string
						// find 1st non-white space
		while ( p < pEnd && *p < '!' || *p > '~' )
			++p;
		memmove (s, p, pEnd - p + 1); // move to start of string
	}
	return s;
}

//	-----------------------------------------------------------------------
//	strpad
//	Pads the string with trailing space up to the length specified.
//	The length must NOT include the terminating NULL, which will be added
//	at length+1.
//	DANGER! If padded length > allocated length of string, will overwrite
//	memory, and the results will be unpredictable!
//	Returns a pointer to the string.
char* strpad (char* s, short const sLen)
{
	register short i = (short)strlen(s);	// length
	register char *p = s + i;			// point to end

	while ( i < sLen )
	{
		*p = ' ';
		++p;
		++i;
	}
	return s;
}

//	-----------------------------------------------------------------------
//	strtolower
// 	Converts all chars in a string to lower case.
void strtolower (char* str)
{
	register char *p = str;
	while (*p)
		*(p++) = (char)tolower(*p);
}

//	-----------------------------------------------------------------------
//	strcmplc
//	"String Compare Lower-Case": Case-insensitive string comparison.
//	Compares only the first 80 characters of the strings.
//	Returns negative, zero, or positive as with strcmp.
int strcmplc (char const* str1, char const* str2)
{
	short const maxLen = 80;
	char s1[maxLen + 1], s2[maxLen + 1];

	// copy string 1
	register short length = (short) strlen (str1);
	length = ( length > maxLen ? maxLen : length );
	strncpy(s1, str1, length);
	s1[length] = '\0';
	::strtolower (s1);
	// copy string 2
	length = (short) strlen(str2);
	length = ( length > maxLen ? maxLen : length );
	strncpy(s2, str2, length);
	s2[length] = '\0';
	::strtolower (s2);
	// return result of compare
	return strcmp(s1, s2);
}

//	-----------------------------------------------------------------------
//	CopyNTCStringList
//	Copy a null-terminated list of C strings.
// 	Expects list to end in a null string,
//	that is, fromList[listSize] = 0;
void CopyNTCStringList (
	char** &toList,				//   destination list
	char const * const * const fromList,	//   source list
	int const listSize)			//   number of C strings
{
	// allocate and initialize a null-terminated list
	toList = new char* [listSize + 1];
	char** pTo = toList;
	char const * const * const pEnd = toList + listSize;
	while ( (char**)pTo <= (char**)pEnd )
		*(pTo++) = 0;

	// allocate and copy each item in the list
	pTo = toList;
	char const * const * pFrom = fromList;
	while ( (char**)pTo < (char**)pEnd )
		CToCString ( *(pTo++), *(pFrom++) );
}

//	----------------------------------------------------------------------
//	DeleteNTCStringList
//	Deletes memory for a null-terminated list of strings.
//	Sets list to NULL.
void DeleteNTCStringList (
	char** &list)
{
	if ( list )
	{
		char** p = list;
		while ( *p )
			delete [] *(p++);
		delete [] list;
		list = 0;
	}
}


//--- end of file ---

