//===============================================================
// mmaker.cpp	- makefileMaker Class for VIDE
//
// Copyright (C) 1995-2003  Bruce E. Wampler
// Version 2.00	date: 18 Mar 2003 12:15:56
//
// This file is part of the V C++ GUI Framework, and is covered
// under the terms of the GNU Library General Public License,
// Version 2. This library has NO WARRANTY. See the source file
// vapp.cxx for more complete information about license terms.
//===============================================================

#include <v/vnotice.h>
#include <v/vos.h>
#include "mmaker.h"
#include "videapp.h"
#include "videlib.h"


#define LINE_CONTINUES	'\\'

    static const char SecProjOpts[] = "//% Section 1 - PROJECT OPTIONS";
    static const char SecProjOptsEnd[] = "//%end-proj-opts";
    static const char SecMakefile[] = "//% Section 2 - MAKEFILE";
    static const char SecOptions[] = "//% Section 3 - OPTIONS";
    static const char SecOptionsEnd[] = "//%end-options";
    static const char SecHomeV[] = "//% Section 4 - HOMEV";
    static const char SecTarget[] = "//% Section 5  - TARGET FILE";
    static const char SecSrcFiles[] = "//% Section 6  - SOURCE FILES";
    static const char SecSrcFilesEnd[] = "//%end-srcfiles";
    static const char SecCompiler[] = "//% Section 7  - COMPILER NAME";
    static const char SecIncDirs[] = "//% Section 8  - INCLUDE DIRECTORIES";
    static const char SecIncDirsEnd[] = "//%end-include-dirs";
    static const char SecLibDirs[] = "//% Section 9 - LIBRARY DIRECTORIES";
    static const char SecLibDirsEnd[] = "//%end-library-dirs";
    static const char SecDefs[] = "//% Section 10  - DEFINITIONS";
    static const char SecDefsPoolEnd[] = "//%end-defs-pool";
    static const char SecDefsEnd[] = "//%end-defs";
    static const char SecCFlags[] = "//% Section 11  - C FLAGS";
    static const char SecLibFlags[] = "//% Section 12  - LIBRARY FLAGS";
    static const char SecSrcDir[] = "//% Section 13  - SRC DIRECTORY";
    static const char SecObjDir[] = "//% Section 14  - OBJ DIRECTORY";
    static const char SecBinDir[] = "//% Section 15 - BIN DIRECTORY";
    static const char SecUserTargets[] = "//% Section 16 - USER TARGETS";
    static const char SecUserTargetsEnd[] = "//%end-user-targets";
    static const char SecLibFiles[] = "//% Section 17 - LIBRARY FILES";
    static const char SecLibFilesEnd[] = "//%end-library-files";


//==================>>> makefileMaker::makefileMaker <<<=========================
  makefileMaker::makefileMaker ()
  {
    // changed from ptrs to auto_ptrs - Tom Hilinski
    projOpts.reset ( new vSList );
    incDirs.reset ( new vSList );
    libDirs.reset ( new vSList );
    libFiles.reset ( new vSList(256) );
    srcFiles.reset ( new vSList(1000) );
    defsPool.reset ( new vSList(256) );
    curDefs.reset ( new vSList(256) );
    curOpts.reset ( new vSList(256) );
    rules.reset ( new vSList(12000) );	   // changed from 1200 - Tom Hilinski
    userTargets.reset ( new vSList(1000) ); // changed from 256  - Tom Hilinski

    setDefaults();
    myCmdw = ((videApp*)theApp)->GetMsgWindow();

  }

//======================>>> makefileMaker::srcN2ObjN <<<=========================
  const char* makefileMaker::srcN2ObjN (const char* n, const char *ext)
  {
    static char objn[maxFileNameSize];
    char *curExt;
    strcpy(objn, n);
    int len = strlen(objn);
    int ix;

    // replace .cpp with .o
    for (ix = len - 1 ; ix >= 0 ; --ix)
      {
	if (objn[ix] == '.')
	  {
	    curExt = &objn[ix+1];
	    break;
	  }
      }

    objn[ix] = 0;

#ifdef V_VersionWindows
    if (stricmpV(curExt,"rc") == 0)
      {
	if (cc == "BCC32")		// borland uses .res
	    strcat(objn,".res");
	else
	    strcat(objn,ext);		// gcc uses .o
      }
    else
#endif
	strcat(objn,ext);
    return stripDir(objn);
  }

//======================>>> makefileMaker::catSrc <<<=========================
  const char* makefileMaker::catSrc (
	const char* src,
	const char *file)
  {
    // this will allow user to use something other that $(Src)
    // for source files.
#ifdef USE_SRC_INMAKE
    static char srcfile[maxFileNameSize];
    if (*file && *file == '=')
	return ++file;
    strcpy(srcfile,src);
    strcat(srcfile,file);
    return srcfile;
#else				// changed in 2.00
    if (*file && *file == '=')
	return ++file;
    return file;
#endif
  }

//======================>>> makefileMaker::eraseAll <<============================
  void makefileMaker::eraseAll()
  {
    projOpts->erase();
    incDirs->erase();
    libDirs->erase();
    libFiles->erase();

    srcFiles->erase();
    defsPool->erase();
    curDefs->erase();
    curOpts->erase();
    rules->erase();
    userTargets->erase();

    BinDir.clear ();
    cc.clear ();
    cFlags.clear ();
    ctags.clear ();
    debugSwitches.clear ();
    exeName.clear ();
    homeV.clear ();
    ldFlags.clear ();
    makeName.clear ();
    ObjDir.clear ();
    SrcDir.clear ();
 }

//======================>>> makefileMaker::setDefaults <<============================
  void makefileMaker::setDefaults ()
  {
    //erase previous definitions
    eraseAll();

    // init each one
    makeName = "Makefile.v";
    vOS vos;
    char buffer[maxFileNameSize];
#ifdef V_VersionX
    if (vos.vGetEnvVal("HOMEV",buffer,maxFileNameSize) < 1)
	homeV = "$(HOME)/v";
    else
    	homeV = buffer;
    exeName = "";
    ctags = "*";
    ldFlags = "";
#else
    if (vos.vGetEnvVal("HOMEV",buffer,maxFileNameSize) < 1)
	homeV = "C:/v";
    else
    	homeV = buffer;
    exeName = "";
    ctags = "*";
    ldFlags = "-mwindows";
    debugSwitches = "";
#endif
    cc = "g++";
    cFlags = "-O";
    ObjDir = ".";
    BinDir = ".";
    SrcDir = ".";
//    libDirs->insert(-1,"$(HOMEV)/lib");	// handled by wizard now
//    incDirs->insert(-1,"$(HOMEV)/include");
    changed = false;		// not changed yet
  }

//======================>>> makefileMaker::stripDir <<<============================
  const char * makefileMaker::stripDir (const char* n)
  {
    for (int ix = strlen(n) - 1 ; ix >= 0 ; --ix)
      {
	if ( IsPathSeparator (n[ix]) )
	  {
	    return &n[ix+1];
	  }
      }
    return n;
 }

//======================>>> makefileMaker::addFile <<<============================
  bool makefileMaker::addFile (const char *srcN, bool addRelPath)
  {
    //adds a new source file to the list
    // Filter name here... (add a behavior switch eventually)
    // Work entirely in Unix / path conventions

    if (!*srcN)
	return false;

    myCmdw = ((videApp*)theApp)->GetMsgWindow();

    string srcFile = srcN;			// make a strings to work with
    string dot = myCmdw->getProjectName();

    string relPath = getRelativePath(dot, srcFile);
    string incPath = ".";

    if (relPath == ".")
	relPath = stripDir(srcFile.c_str());
    else
      {
	incPath = relPath;
	relPath += '/';
	relPath += stripDir(srcFile.c_str());
      }

    fixSeparators(relPath);

    for (int ix = 0 ; srcFiles->list[ix] != 0 ; ++ix)
	if (strcmp(srcFiles->list[ix], relPath.c_str()) == 0)
	    return false; 	//don't add a obj file twice

    if (srcFiles->insert(-1,relPath.c_str()) < 0)
        return false;

    if (incPath != ".")
	(void) addInclude((const char*) incPath.c_str());	// auto add inc paths

    changed = true;
    return true;
  }

//======================>>> makefileMaker::addLibFile <<<============================
  bool makefileMaker::addLibFile (const char *libN)
  {
    // adds a new library file to the list
    // Filter name here... (add a behavior switch eventually)
    // Work entirely in Unix / path conventions

    if (!*libN)
	return false;

    myCmdw = ((videApp*)theApp)->GetMsgWindow();

    string libFile = libN;			// make a strings to work with
    string dot = myCmdw->getProjectName();

    string filePart = getFilePart(libFile);

    for (int ix = 0 ; libFiles->list[ix] != 0 ; ++ix)
	if (strcmp(libFiles->list[ix], filePart.c_str()) == 0)
	    return false; 	//don't add a obj file twice

    if (libFiles->insert(-1,filePart.c_str()) < 0)
        return false;

    changed = true;
    return true;
  }

//======================>>> makefileMaker::addInclude <<<============================
  bool makefileMaker::addInclude(const char *incD)
  {
    //adds a new include path to the list
    // Work entirely in Unix / path conventions

    if (!*incD)
	return false;

    myCmdw = ((videApp*)theApp)->GetMsgWindow();

    string incDir = incD;

    fixSeparators(incDir);		// force to / (or \ for BCC)

    for (int ix = 0 ; incDirs->list[ix] != 0 ; ++ix)
	if (strcmp(incDirs->list[ix], incDir.c_str()) == 0)
	    return false; 	//don't add a obj file twice

    if (incDirs->insert(-1,incDir.c_str()) < 0)
        return false;
    changed = true;
    return true;
  }

//======================>>> makefileMaker::findRule <<<============================
  void makefileMaker::findRule(const char* name, const char* comp)
  {
    // finds the rules to build object file pointed by name

    vOS vos;		// To execute g++

    std::string command;		// command line
    std::string hvbuff;
    // Increased buffer size substantially! Tom Hilinski, Dec2002
    static short const currlineMaxLength = 4096;
    char currlin[currlineMaxLength];

    command = comp;
    // add include directories
    for (int ix = 0 ; incDirs->list[ix] != 0 ; ++ix)
      {
	std::string incdir = incDirs->list[ix];
	char const * const hvp = strstr(incdir.c_str(),"$(HOMEV)/");

	if (hvp != 0)
	  {
	    // Have $(HOMEV) in the path, so we must replace the
	    // $(HOMEV) with the real homeV path.
	    hvbuff = homeV;
	    hvbuff += hvp + strlen("$(HOMEV)");
	    incdir = hvbuff;
	  }

	command += " -I";
	command += incdir;
	// if (strlen(command) > 480)
	//    break;
      }
    command += ' ';
    command += cFlags;	// include any command-line defines
    command += " -MM ";

//    hvbuff = SrcDir;
//    hvbuff += UNIX_PATH_SEPARATOR;	// consistancy demands Unix-style
    hvbuff = name;
    command += fileQuote (hvbuff);

    static char makedep[] = "videtmp0.vtm";
    static char makeerr[] = "videtmp1.vtm";

    vos.vDeleteFile(makedep);	// be sure these are gone...
    vos.vDeleteFile(makeerr);

    myCmdw = ((videApp*)theApp)->GetMsgWindow();

    if (vos.vRunProcess(command.c_str(), makedep, makeerr, 1, 1) != 0)
      {
CANTMAKE:
	myCmdw->AddLine ( command.c_str() );
	vos.vDeleteFile(makedep);
        ifstream em(makeerr);
        if (em)
          {
            while(em.getline(currlin, currlineMaxLength,'\n'))
		myCmdw->AddLine(currlin);
	    myCmdw->AddLine("* Build of Makefile failed! Possible causes include:");
	    myCmdw->AddLine("  1. Missing header or source files or include paths.");
	    myCmdw->AddLine("  2. Changing directory structure or moving files.");
	    myCmdw->AddLine("  3. Moving the VIDE project file to a new directory.");
	    myCmdw->AddLine("--------------");
            em.close();
          }
        vos.vDeleteFile(makeerr);
	return;
      }

    ifstream deps(makedep);
    if (!deps)
	goto CANTMAKE;


    while ( deps.getline(currlin, currlineMaxLength, '\n'))
      {
        if (rules->insert(-1,currlin) < 0)
            break;
      }

    deps.close();
    vos.vDeleteFile(makedep);
    vos.vDeleteFile(makeerr);
  }

//======================>>> makefileMaker::fixTabs <<<============================
  void makefileMaker::fixTabs (
	char* fixed,
	char* orig)
  {
    while (*orig)
      {
	if (*orig == '\\' && *(orig+1) =='t')	// replace with real tab
	  {
	    *fixed++ = '\t';
	    orig++; orig++;
	  }
	else
	    *fixed++ = *orig++;
      }
    *fixed = 0;
  }

//====================>>> makefileMaker::saveMakefile <<<=========================
  void makefileMaker::saveMakefile ()
  {

    bool isBorland;		// set up Borland stuff

    char resName[maxFileNameSize];

    resName[0] = 0;

    libsBCC.erase();

    //builds the file named makefile using infos stored in project

    // Now, change directory, drive

    myCmdw = ((videApp*)theApp)->GetMsgWindow();
    changeDir(myCmdw->getProjectName());	// have to run from proj dir

    int ix = makeName.length();
    while (ix > 0)
      {
        if ( IsPathSeparator (makeName[ix]) )
          {
            break;
          }
        --ix;
      }
    if (ix > 0)
      {
	vOS vos;
	std::string dirPath = makeName;	// copy the makefile name
#ifdef V_VersionWindows
        if (dirPath[1] == ':')	    // a drive
	  {
	    int drive = dirPath[0]; // need to change drive, too.
	    if (drive > 'a')
		drive = drive - ' ';
	    vos.vChDrive(drive-'A');
	  }
#endif
        vos.vChDir( dirPath.c_str() );			// change dir
      }


    ofstream mkf;
    // always write lf only makefiles...
    mkf.open(makeName.c_str(), std::ios::binary | std::ios::out); // where to write makefile

    //header
    char date[20];
    char time[40];

    vGetLocalTime(time);
    vGetLocalDate(date);

    isBorland = (cc == "BCC32");

    myCmdw = ((videApp*)theApp)->GetMsgWindow();

    if (isBorland)
	myCmdw->AddLine("Building Borland Makefile...");
    else
	myCmdw->AddLine("Building Makefile -- Running compiler to generate dependencies...");


    mkf << "#=======================================================================\n";
    mkf << "#@V@:Note: File automatically generated by ";
    mkf << VIDE_VERS << " (" << cc << ").\n";
    mkf << "# Generated " << time << " " << date << '\n';
    mkf << "# This file regenerated each time you run VIDE, so save under a\n";
    mkf << "#    new name if you hand edit, or it will be overwritten.\n";
    mkf << "#=======================================================================\n\n";
    mkf << "# Standard defines:\n";

    if (isBorland)
      {
#ifdef V_VersionWindows
	mkf << ".AUTODEPEND\n\n"
	    << "# Borland C++ tools\n"
	    << "IMPLIB\t= $(BCCROOT)\\bin\\Implib\n"
	    << "ILINK32\t= $(BCCROOT)\\bin\\ILink32\n"
	    << "TLIB\t= $(BCCROOT)\\bin\\TLib\n"
	    << "BRC32\t= $(BCCROOT)\\bin\\Brc32\n"
	    << "TASM32\t= $(BCROOT)\\bin\\Tasm32\n"
	    << "CC  \t= $(BCCROOT)\\bin\\" << cc << "\n\n";
	if (*((videApp*)theApp)->getCCPath())
	  {
	    mkf << "BCCROOT\t= " <<
	     fileQuote(fixBackSlash(((videApp*)theApp)->getCCPath(),isBorland))
	     << "\n\n";
	  }
	else
	    mkf << "BCCROOT\t= C:\\borland\\bcc55\n\n";
#endif
      }
    else		// gnu stuff
      {
	mkf << "CC  \t=\t" << cc << "\n";
	mkf << "WRES\t=\twindres\n";
        if (cc == "g++")
	    mkf << "CCOMP\t=\tgcc\n";
      }

    // HomeV
    mkf << "HOMEV\t=\t" << fixBackSlash(homeV,isBorland) << '\n';

    //Dirs
    if (isBorland)
	mkf << "VPATH\t=\t$(HOMEV)\\include\n";
    else
	mkf << "VPATH\t=\t$(HOMEV)/include\n";

    mkf << "oDir\t=\t" << fileQuote(fixBackSlash(ObjDir, isBorland)) << '\n'; //Obj Dir
    mkf << "Bin\t=\t" << fileQuote(fixBackSlash(BinDir, isBorland)) << '\n'; //Bin Dir
//    mkf << "Src\t=\t" << fileQuote(fixBackSlash(SrcDir, isBorland)) << '\n'; //Src Dir

    mkf << "libDirs\t=";

    int nxtld = libDirs->size();

    if (isBorland && nxtld > 0)
	mkf << "\t-L";

    for (int ixx = 0 ; ixx < nxtld ; ++ixx)
      {
	if (isBorland)		// put out ; separated list
	  {
	    mkf << fixBackSlash(libDirs->list[ixx],isBorland);
	    if (ixx + 1 < nxtld)
		mkf << ";";
	  }
	else
	  {
	    mkf << "\t-L" << fixBackSlash(libDirs->list[ixx],isBorland);
	    if (ixx + 1 < nxtld)
		mkf << " \\";
	    mkf << '\n';
	  }
      }
    if (isBorland)
	mkf << '\n';
    mkf << std::endl;

    mkf << "incDirs\t=";

    int nxtid = incDirs->size();

    if (isBorland && nxtid > 0)
	mkf << "\t-I";

    for (int ixa = 0 ; ixa < nxtid ; ++ixa)
      {
	if (isBorland)		// put out ; separated list
	  {
	    mkf << fixBackSlash(incDirs->list[ixa],isBorland);
	    if (ixa + 1 < nxtid)
		mkf << ";";
	  }
	else
	  {
	    mkf << "\t-I" << fixBackSlash(incDirs->list[ixa],isBorland);
	    if (ixa + 1 < nxtid)
		mkf << " \\";
	    mkf << '\n';
	  }
      }
    if (isBorland)
	mkf << '\n';
    mkf << '\n';

    //ld flags
//    if (isBorland)
//      {
//	mkf << "LD_FLAGS\t=\t" << fixBCCLibs() << '\n';
//      }
//    else
	mkf << "LD_FLAGS =\t" << ldFlags << '\n';

    // LIBS switch
    if (!isBorland)
	mkf << "LIBS\t=\t" << getLibFiles() << '\n';

    //C_FLAGS & paths to includes

    mkf << "C_FLAGS\t=\t" << fixBackSlash(cFlags,isBorland);

    //defines appended to C_FLAGS on separate lines
    int nxtcd = curDefs->size();

    for (int ixb = 0 ; ixb < nxtcd ; ++ixb)
      {
	mkf << "\\\n\t" << curDefs->list[ixb];
      }
    mkf << "\n\n";

    //sources
    int nxtof = srcFiles->size();

    mkf << "SRCS\t=";

    for (int ixc = 0 ; ixc < nxtof ; ++ixc)
      {
	if (isBorland)
	    mkf << "\\\n\t" <<
	      catSrc("$(Src)\\",fixBackSlash(srcFiles->list[ixc], isBorland));
	else
	    mkf << "\\\n\t" << catSrc("$(Src)/", srcFiles->list[ixc]);
      }
    mkf << "\n\n";

    //objects

    mkf << "EXOBJS\t=";

    for (int ixd = 0 ; ixd < nxtof ; ++ixd)
      {
	const char *objn;
	if (isBorland)
	  {
	    objn = fixBackSlash(srcN2ObjN(srcFiles->list[ixd],".obj"),isBorland);
	    mkf << "\\\n\t$(oDir)\\" << objn;

	    if (strstr(objn,".res") != 0)
	      {
		if (!*resName)
		  {
		    strcpy(resName,"$(oDir)\\");
		    strcat(resName,objn);
		  }
		else
		  {
		    strcat(resName,"+$(oDir)\\");
		    strcat(resName,objn);
		  }
	      }
	  }
	else
	  {
	    objn = srcN2ObjN(srcFiles->list[ixd],".o");
	    mkf << "\\\n\t$(oDir)/" << objn;
	  }

      }
    mkf << "\n\n";

    mkf << "ALLOBJS\t=\t$(EXOBJS)" << std::endl;
    if (isBorland)
      {
	mkf << "BCCJUNK\t=\t*.il* *.csm *.tds\n";
	mkf << "ALLBIN\t=\t$(Bin)\\" << exeName << '\n';
	mkf << "ALLTGT\t=\t$(Bin)\\" << exeName << "\n\n";
      }
    else
      {
	mkf << "ALLBIN\t=\t$(Bin)/" << exeName << '\n';
	mkf << "ALLTGT\t=\t$(Bin)/" << exeName << "\n\n";
      }


    // Options

    mkf << "# User defines:\n";
    for (int ixe = 0 ; curOpts->list[ixe] != 0 ; ++ixe)
      {
      	char buff[256];
	fixTabs(buff,curOpts->list[ixe]);
	mkf << buff << '\n';
      }
    mkf << '\n';

    mkf << "#@# Targets follow ---------------------------------\n\n";
    //target file
    //all:
    mkf << "all:\t$(ALLTGT)\n\n";
    //objs:
    mkf << "objs:\t$(ALLOBJS)\n\n";

    //cleanobjs:
    mkf << "cleanobjs:\n\trm -f $(ALLOBJS)\n\n";

    // cleanbin
    if (isBorland)
	mkf << "cleanbin:\n\trm -f $(ALLBIN) $(BCCJUNK)\n\n";
    else
	mkf << "cleanbin:\n\trm -f $(ALLBIN)\n\n";

    // clean
    mkf << "clean:\tcleanobjs cleanbin\n\n";

    // cleanall
    mkf << "cleanall:\tcleanobjs cleanbin\n\n";

    mkf << "#@# User Targets follow ---------------------------------\n\n";

    for (int ixf = 0 ; userTargets->list[ixf] != 0 ; ++ixf)
      {
      	char buff[256];
	fixTabs(buff,userTargets->list[ixf]);
	mkf << buff << '\n';
      }
    mkf << '\n';

    mkf << "#@# Dependency rules follow -----------------------------\n\n";
    //dependency rules

    // allow user to override the all: target with a #all comment
    if (!(userTargets->list[0] && strstr(userTargets->list[0],"#all")!=0))
      {
        bool isStaticLib = false;
        char ext[32];

	// check for a static library
	splitFileName(exeName.c_str(),0,0,ext);	// get the extention
	if (stricmpV(ext,".a") == 0
#ifdef V_VersionWindows
            || stricmpV(ext,".lib") == 0 ||
	    stricmpV(ext,".LIB") == 0
#endif
           )
	    isStaticLib = true;		// found a static lib

	if (isBorland)
	  {
#ifdef V_VersionWindows
	    mkf << "$(Bin)\\" << exeName << " : $(EXOBJS)\n";

	    if (isStaticLib)
	      {
		mkf << "  rm -f $<\n";
		mkf << "  $(TLIB) $< @&&|\n";
		for (int ixg = 0 ; srcFiles->list[ixg] != 0 ; ixg++)
		  {
		    if (ixg != 0)
			mkf << " &\n";
		    mkf << " +$(oDir)\\" <<
			srcN2ObjN(srcFiles->list[ixg],".obj");
		  }
		mkf << " \n|\n\n";

	      }
	    else		// standard linker commands
	      {
		mkf << "  $(ILINK32) @&&|\n" << " $(LD_FLAGS) $(libDirs) +\n";

		mkf << "$(BCC32STARTUP)";

		for (int ixg = 0 ; srcFiles->list[ixg] != 0 ; ixg++)
		  {
		    const char* objn = srcN2ObjN(srcFiles->list[ixg],".obj");
		    if (strstr(objn,".res") == 0)
		      {
			mkf << " +\n$(oDir)\\" << 
			   srcN2ObjN(srcFiles->list[ixg],".obj");
		      }
		  }

		mkf << "\n$<,$*\n";

		// OK, scan the libs list to output BCC lib files
		for (int ixa = 0 ; libFiles->list[ixa] != 0 ; ixa++)
		  {
		    if (ixa != 0)
			mkf << " +\n";
		    if (strstr(libFiles->list[ixa],".lib") != 0 ||
			strstr(libFiles->list[ixa],".LIB") != 0)
		        mkf << libFiles->list[ixa];	// already has .lib
		    else
			mkf << libFiles->list[ixa] << ".lib";
		  }

		mkf << "\n$(BCC32DEF)\n";	// 1.16 - allow DEF files!
		if (*resName)		// had a .res file
		  {
		    mkf << resName << "\n\n|\n";
		  }
		else
		    mkf << "\n\n|\n";
	      }
#endif
	  }
	else		// gnu compatible;
	  {
	    mkf << "$(Bin)/" << exeName << ": $(EXOBJS)\n";
	    if (isStaticLib)
	      {
		mkf << "\trm -f $(Bin)/" << exeName << '\n';
		mkf << "\tar cr $(Bin)/" << exeName << " $(EXOBJS)\n";
		mkf << "\tranlib $(Bin)/" << exeName << '\n';
	      }
	    else
	      {
		mkf << "\t$(CC) -o $(Bin)/"
		<< exeName << " $(EXOBJS) $(incDirs) $(libDirs) $(LD_FLAGS) $(LIBS)\n";
	      }
	  }
      }

    if (isBorland)
      {
	// now count on autodependency for borland
	for (int ixg = 0 ; srcFiles->list[ixg] != 0 ; ixg++)
	  {
	    mkf << "$(oDir)\\" <<srcN2ObjN(srcFiles->list[ixg],".obj");
	    mkf << " : " << catSrc("$(Src)\\", srcFiles->list[ixg]) << '\n';

	    // check for .rc files
	    if (strstr(srcFiles->list[ixg],".rc") != 0
		|| strstr(srcFiles->list[ixg],".RC") != 0)
	      {
		mkf 	<< "  $(BRC32) -r $(incDirs) -fo$@ " <<
			catSrc("$(Src)\\", srcFiles->list[ixg]) << "\n\n";

	      }
	    else		// it is a .rc file
	      {
		mkf 	<< "  $(CC) -c $(C_FLAGS) $(incDirs) -o$@ "
			<< catSrc("$(Src)\\",srcFiles->list[ixg]) << "\n\n";
	      }
	  }
      }
    else			// NOT Borland
      {
	rules->erase();	// fix by chorn - 03/08/99

	for (int ixg = 0 ; srcFiles->list[ixg] != 0 ; ixg++)
	  {
	    // make GUI more responsive here - Tom Hilinski, Dec2002:
            theApp->CheckEvents();

#ifdef V_VersionWindows
	    if (strstr(srcFiles->list[ixg],".rc") != 0
		|| strstr(srcFiles->list[ixg],".RC") != 0)
	      {
		mkf << "\n$(oDir)/" << srcN2ObjN(srcFiles->list[ixg],".o");
		mkf << " : " << catSrc("$(Src)/",srcFiles->list[ixg]) << '\n';
		mkf << "\t$(WRES) " << catSrc("$(Src)/",srcFiles->list[ixg]);
		mkf << " $(oDir)/" << srcN2ObjN(srcFiles->list[ixg],".o");
		mkf << '\n';
	      }
	    else
#endif
	    if (needCFile(srcFiles->list[ixg]))	// mix c++/c?
	      {
		findRule(srcFiles->list[ixg],"gcc");
	      }
	    else
		findRule(srcFiles->list[ixg],cc.c_str());
	  }

	int lim = rules->size();
	for (int ixh = 0 ; ixh < lim ; ++ixh)
	  {
	    
	    bool needc = needCFile(rules->list[ixh]);	// C++/c mix?

	    mkf << "\n$(oDir)/" << rules->list[ixh] << '\n';
	    // check for continuation lines
	    while ( *(rules->list[ixh] + strlen(rules->list[ixh])-1) ==
		    LINE_CONTINUES)
	      {
		++ixh;
		if (rules->list[ixh])
		    mkf << rules->list[ixh] << '\n';
	      }
	    if (needc)
		mkf << "\t$(CCOMP) $(C_FLAGS) $(incDirs) -c -o $@ $<\n";
	    else
		mkf << "\t$(CC) $(C_FLAGS) $(incDirs) -c -o $@ $<\n";
	  }
      }

    mkf.close();

    std::string buffer = makeName;
    buffer += " (";
    buffer += time;
    buffer += " ";
    buffer += date;
    buffer += ") - Makefile saved.";
    myCmdw = ((videApp*)theApp)->GetMsgWindow();
    myCmdw->AddLine ( buffer.c_str() );
  }

//======================>>> makefileMaker::getLibFiles <<<============================
  string makefileMaker::getLibFiles()
  {
    // This will take the list of libraries in the libFiles list and
    // convert them to a string of -l's with the lib part strip suitable
    // for gnu ld input.

    static char fbase[maxFileNameSize];
    string libs = "";

    // OK, scan the libs list to output BCC lib files
    for (int ixa = 0 ; libFiles->list[ixa] != 0 ; ixa++)
      {
	if (ixa > 0)		// need blank when past first one
	    libs += " -l";
	else
	    libs += "-l";

	splitFileName(libFiles->list[ixa],0,fbase,0);	// get base

	if (strstr(fbase,"lib") != fbase)
	    libs += fbase;	// something not foo
	else
	  {
	    string sbase(&fbase[3]);
	    libs += sbase;	// strip the lib
	  }
      }
    return libs;
  }

//======================>>> makefileMaker::needCFile <<<============================
  bool makefileMaker::needCFile(const char* cF)
  {
    const char* cp = strstr(cF,".c");

    if (strcmp(cc.c_str(),"g++") != 0)	// can't need if not g++
	return false;

    if (!cp)				// find a a ".C"
	cp = strstr(cF,".C");
    if (!cp)				// no .c's
	return false;
    if (*(cp+2) == ' ' || *(cp+2) == 0)	// must be .c only
	return true;
    return false;
  }

//======================>>> makefileMaker::saveProject <<<============================
  void makefileMaker::saveProject (char const * const nameIn)
  {

    char n[300];	// project file name

    if (!changed)
	return;

    if (strlen(nameIn) > 296)
	strcpy(n,"BADFILE.tmp");
    else
	strcpy(n,nameIn);

    int len = strlen(n);
    int ix;

    for (ix = len - 1 ; ix >= 0 ; --ix)
      {
	if ( n[ix] == '.' || IsPathSeparator (n[ix]) )
	    break;
      }
    if (n[ix] != '.')	// user didn't supply extension
	strcat(n,".vpj");

    ofstream prjf(n);

    //header
    char buff[40];
    char date[20];
    vGetLocalTime(buff);
    vGetLocalDate(date);

    prjf << "//=======================================================================\n";
    prjf << "//@V@:Note: Project File generated by ";
    prjf << VIDE_VERS << '\n';
    prjf << "// Generated " << buff << " " << date << '\n';
    prjf << "// CAUTION! Hand edit only if you know what you are doing!\n";
    prjf << "//=======================================================================\n\n";

    // Project Options
    prjf << SecProjOpts << std::endl;

    projOpts->erase();		// erase all options
    if (ctags.length() > 0)
      {
	std::string buff = "ctags:";
	buff += ctags;
	projOpts->insert(-1, buff.c_str() );
      }

    if (debugSwitches.length() > 0)
      {
	std::string buff = "debugSwitches:";
	buff += debugSwitches;
	projOpts->insert(-1,buff.c_str() );
      }

    for (int nxtOpts = 0 ; projOpts->list[nxtOpts] != 0 ; ++nxtOpts)
	prjf << projOpts->list[nxtOpts] << std::endl;

    prjf << SecProjOptsEnd << std::endl << std::endl;

    // Makefile name
    prjf << SecMakefile << std::endl;
    prjf << makeName << std::endl << std::endl;

    // Options (User defines)
    prjf << SecOptions << std::endl;

    for (int nxtOptsa = 0 ; curOpts->list[nxtOptsa] != 0 ; ++nxtOptsa)
	prjf << curOpts->list[nxtOptsa] << std::endl;

    prjf << SecOptionsEnd << std::endl << std::endl;

    // HOMEV
    prjf << SecHomeV << std::endl;
    prjf << homeV << std::endl << std::endl;

    // 1) name of exe target file
    prjf << SecTarget << std::endl;
    prjf << exeName << std::endl << std::endl;

    // 2) object files
    prjf << SecSrcFiles << std::endl;
    for (int ixa = 0 ; srcFiles->list[ixa] != 0 ; ++ixa)
      {
	prjf <<  srcFiles->list[ixa] << std::endl;
      }
    prjf << SecSrcFilesEnd << std::endl << std::endl;

    // 3) compiler name
    prjf << SecCompiler << std::endl;
    prjf << cc << std::endl << std::endl;

    // 4) include dirs
    prjf << SecIncDirs << std::endl;

    for (int nxtid = 0 ; incDirs->list[nxtid] != 0 ; ++nxtid)
	prjf << incDirs->list[nxtid] << std::endl;

    prjf << SecIncDirsEnd << std::endl << std::endl;

    // 5) lib dirs
    prjf << SecLibDirs << std::endl;
    for (int ld = 0 ; libDirs->list[ld] != 0 ; ++ld)
	prjf << libDirs->list[ld] << std::endl;
    prjf << SecLibDirsEnd << std::endl << std::endl;

    // 6) defs
    prjf << SecDefs << std::endl;
    for (int ixb = 0 ; defsPool->list[ixb] != 0 ; ++ixb)
	prjf << defsPool->list[ixb] << std::endl;
    prjf << SecDefsPoolEnd << std::endl;

    for (int ixc = 0 ; curDefs->list[ixc] != 0 ; ++ixc)
	prjf << curDefs->list[ixc] << std::endl;
    prjf << SecDefsEnd << std::endl << std::endl;

    // 7) cflag
    prjf << SecCFlags << std::endl;
    prjf << cFlags << std::endl << std::endl;

    // 8) ldflag
    prjf << SecLibFlags << std::endl;
    prjf << ldFlags << std::endl << std::endl;

    // src directory
    prjf << SecSrcDir << std::endl;
    prjf << SrcDir << std::endl << std::endl;

    // 9) object directory
    prjf << SecObjDir << std::endl;
    prjf << ObjDir << std::endl << std::endl;

    //10) bin directory
    prjf << SecBinDir << std::endl;
    prjf << BinDir << std::endl << std::endl;
    // User Targets
    prjf << std::endl << "//% User targets section. Following lines will be" << std::endl;
    prjf << "//% inserted into Makefile right after the generated cleanall target." << std::endl;
    prjf << "//% The Project File editor does not edit these lines - edit the .vpj" << std::endl;
    prjf << "//% directly. You should know what you are doing." << std::endl;

    prjf << SecUserTargets << std::endl;
    for (int lda = 0 ; userTargets->list[lda] != 0 ; ++lda)
	prjf << userTargets->list[lda] << std::endl;
    prjf << SecUserTargetsEnd << std::endl << std::endl;

    prjf << SecLibFiles << std::endl;
    for (int lda = 0 ; libFiles->list[lda] != 0 ; ++lda)
	prjf << libFiles->list[lda] << std::endl;
    prjf << SecLibFilesEnd << std::endl << std::endl;

    prjf.close();

    saveMakefile();		// and always save the makefile, too.
    changed = false;
  }

//===================>>> makefileMaker::getSectionEntry <<<===========================
  bool makefileMaker::getSectionEntry (
  	ifstream & prjf,
  	const char* head,
  	std::string & value)
  {
    // read section head into value
    short const inputLineMaxLength = 4096;
    char skipStr[inputLineMaxLength];
    if (!prjf)
	return false;
    for (prjf.getline(skipStr, inputLineMaxLength, '\n') ;
         prjf ;
         prjf.getline(skipStr, inputLineMaxLength, '\n'))
      {
	char* cp = skipStr;
	while (*cp)			// make cr/lf files ok on Linux systems
	  {
	    if (*cp == '\r' || *cp == '\n')
		*cp = 0;
	    else
		++cp;
	  }
	if (strcmp(skipStr,head) == 0)
	    break;
      }

    // next line will be the entry...

    prjf.getline(skipStr, inputLineMaxLength, '\n') ;	// skip blank lines

    char* cp = skipStr;
    while (*cp)			// make cr/lf files ok on Linux systems
      {
	if (*cp == '\r' || *cp == '\n')
	    *cp = 0;
	else
	    ++cp;
      }

    value = skipStr;			// copy to output

    if (!prjf)
	return false;
    else
	return true;

  }

//===================>>> makefileMaker::getSectionList <<<============================
  bool makefileMaker::getSectionList(
  	ifstream &prjf,
  	const char* head,
  	const char* term,
  	vSList* sl)
  {
    short const inputLineMaxLength = 4096;
    char skipStr[inputLineMaxLength];
    if (!prjf)
	return false;

    if (head && *head)	// allow no head - assumes at right location
      {
	for (prjf.getline(skipStr, inputLineMaxLength, '\n') ;
	     prjf ;
	     prjf.getline(skipStr, inputLineMaxLength, '\n'))
	  {
	    char* cp = skipStr;
	    while (*cp)			// make cr/lf files ok on Linux systems
	      {
		if (*cp == '\r' || *cp == '\n')
		    *cp = 0;
		else
		    ++cp;
	      }
	    if (strcmp(skipStr,head) == 0)
		break;
	  }
      }

    // get list now
    for (prjf.getline(skipStr, inputLineMaxLength, '\n') ;
         prjf ;
         prjf.getline(skipStr, inputLineMaxLength, '\n'))
      {
	char* cp = skipStr;
	while (*cp)			// make cr/lf files ok on Linux systems
	  {
	    if (*cp == '\r' || *cp == '\n')
		*cp = 0;
	    else
		++cp;
	  }
	if (strcmp(skipStr,term) == 0)
	    break;
	if (*skipStr != ' ')
	    if (sl->insert(-1,skipStr) < 0)		// add to end of list
		break;
      }

    if (!prjf)
	return false;
    else
	return true;

  }

//======================>>> makefileMaker::loadProject <<<============================
  bool makefileMaker::loadProject (
	char const * const n)
  {
    //loads infos contained in project file n into makefileMaker member vars

    ifstream prjf(n);
    if (!prjf)
	return false;

    eraseAll();

    // Project Options
    if (!getSectionList(prjf, SecProjOpts, SecProjOptsEnd, projOpts.get() ))
	return false;

    // extract options to useful fields
    int optlim = projOpts->size();
    for (int ixa = 0 ; ixa < optlim ; ++ixa)
      {
	if (strstr(projOpts->list[ixa],"ctags:")!= 0)
	  {
	    ctags = &projOpts->list[ixa][strlen("ctags:")];
	  }
	if (strstr(projOpts->list[ixa],"debugSwitches:")!= 0)
	  {
	    debugSwitches = &projOpts->list[ixa][strlen("debugSwitches:")];
	  }
      }
    projOpts->erase();		// now erase all options to avoid dups

    // Makefile
    if (!getSectionEntry(prjf, SecMakefile, makeName))
      {
	prjf.close();
	return false;
      }

    //0) options (User defines)
    if (!getSectionList(prjf, SecOptions, SecOptionsEnd, curOpts.get() ))
      {
	prjf.close();
	return false;
      }

    // HOMEV
    if (!getSectionEntry(prjf, SecHomeV, homeV))
      {
	prjf.close();
	return false;
      }

    //1) exe name
    if (!getSectionEntry(prjf, SecTarget, exeName))
      {
	prjf.close();
	return false;
      }

    //2) srcs
    if (!getSectionList(prjf, SecSrcFiles, SecSrcFilesEnd, srcFiles.get() ))
      {
	prjf.close();
	return false;
      }

    //3) compiler name
    if (!getSectionEntry(prjf, SecCompiler, cc))
      {
	prjf.close();
	return false;
      }

    //4) include dirs
    if (!getSectionList(prjf, SecIncDirs, SecIncDirsEnd, incDirs.get() ))
      {
	prjf.close();
	return false;
      }

    //5) lib dirs
    if (!getSectionList(prjf, SecLibDirs, SecLibDirsEnd, libDirs.get() ))
      {
	prjf.close();
	return false;
      }

    //6) defs
    if (!getSectionList(prjf, SecDefs, SecDefsPoolEnd, defsPool.get() ))
      {
	prjf.close();
	return false;
      }
    if (!getSectionList(prjf, "", SecDefsEnd, curDefs.get() ))
      {
	prjf.close();
	return false;
      }

    //7) cflag
    if (!getSectionEntry(prjf, SecCFlags, cFlags))
      {
	prjf.close();
	return false;
      }

    //8) libflag
    if (!getSectionEntry(prjf, SecLibFlags, ldFlags))
      {
	prjf.close();
	return false;
      }
    string::size_type pos = ldFlags.find("-l",0);
    if (pos != string::npos)
      {
	vNoticeDialog note(((videApp*)theApp)->GetMsgWindow());
	note.Notice(	"This VIDE project file contains \"-l\" library\n"
			"specifications in the liker flags entry. They are\n"
			"no longer used in VIDE Version 2. You should\n"
			"manually fix this project file by adding the\n"
			"libraries to the Libraries tab in the project\n"
			"editor, and removing them from the linker flags.");
      }

    //  SRC directory
    if (!getSectionEntry(prjf, SecSrcDir, SrcDir))
      {
	prjf.close();
	return false;
      }
    if (SrcDir != ".")
      {
	vNoticeDialog note(((videApp*)theApp)->GetMsgWindow());
	note.Notice(	"This VIDE project file contains a non-'.' Src\n"
			"directory specification which is no longer used\n"
			"in VIDE Version 2. Any makefiles generated by\n"
			"VIDE using this project file will not be correct!\n"
			"You will need to recreate this project using the\n"
			"new VIDE project editor.");
	prjf.close();
	return false;
      }

    // 9) object directory
    if (!getSectionEntry(prjf, SecObjDir, ObjDir))
      {
	prjf.close();
	return false;
      }

    //10) bin directory
    if (!getSectionEntry(prjf, SecBinDir, BinDir))
      {
	prjf.close();
	return false;
      }

    // User targets.
    if (!getSectionList(prjf, SecUserTargets, SecUserTargetsEnd, userTargets.get() ))
      {
	prjf.close();
	return false;
      }

    // ignore if don't have a LibFiles Section
    (void) getSectionList(prjf, SecLibFiles, SecLibFilesEnd, libFiles.get() );

    prjf.close();
    return true;
  }

//============================>>> fixSeparators <<<===========================
  void makefileMaker::fixSeparators(string & fixed)
  {
    // for makefiles, we will stick to / for file separators
    // except for the borland complier

    bool isBorland = (cc == "BCC32");

    string::size_type ix;
    string::size_type len = fixed.length();

    for (ix = 0 ; ix < len ; ++ix)
      {
	if (fixed[ix] == '/')
	  {
	    if (isBorland)
		fixed[ix] = '\\';
	  }
	else if (fixed[ix] == '\\')
	  {
	    if ( ! isBorland )
		fixed[ix] = '/';
	  }
      }
  }

//===========================>>> getRelativePath <<<===========================
  string makefileMaker::getRelativePath(string & dot, string & file2)
  {
    // given a full file path in dot and file2, return the relative
    // path to file2 from dot.

    string relPath;
    string dotPath = getPath(dot);
    string file2Path = getPath(file2);

    string dotFile = getFilePart(dot);
    string fileFile = getFilePart(file2);

    fixSeparators(dotPath);
    fixSeparators(file2Path);

    if (dotPath == file2Path)		// special case
	return ".";

    // Windows - handle on different drives, in which case the relative
    // path is simply the full path to the second file...

    if (dotPath[0] != file2Path[0])	// not equal at all
	return file2Path;

    string::size_type dlen = dotPath.length();	// length of dot's path
    string::size_type flen = file2Path.length();
    string::size_type ix;
    
    for (ix = 0 ; ix < dlen && ix < flen ; ++ix)
      {
	if (dotPath[ix] != file2Path[ix])
	    break;
      }

    // case  dot; /path/file2
    if (ix == dlen)	// ran out of dot path
      {
	// case  dot; /path/file2
	if (ix < flen && IsPathSeparator(file2Path[ix]))
	  {
	    relPath = "." + file2Path.substr(ix);
	    return relPath;
	  }
	// case: something like .../c/dotfile; .../cx/file2
	for (ix = ix ; ix >= 0 ; --ix)
	  {
	    if (IsPathSeparator(file2Path[ix]))
	      {
		relPath = ".." + file2Path.substr(ix);
		return relPath;
	      }
	  }
	// I'm confused - return save alternative
	return file2Path;
      }

    // different at ix

    // count /'s left in dot path
    int seps = 0;
    for (int ip = ix  ; ip < dlen ; ++ip)
      {
	if (IsPathSeparator(dotPath[ip]))
	  {
	    seps++;
	  }
      }
    if (ix < flen)		// dirs, not file in ..
	++seps;
    string dotdot = "..";
    for (int iy = 1 ; iy < seps ; ++iy)
      {
	dotdot = "../" + dotdot;
      }

    if (ix < flen)
      {
	for (int ifp = ix ; ifp >= 0 ; --ifp)
	  {
	    if (IsPathSeparator(file2Path[ifp]))
	      {
		relPath = dotdot + file2Path.substr(ifp);
		return relPath;
	      }
	  }
	// I'm confused - return save alternative
	return relPath;	// return contents on the stack
      }
    relPath = dotdot;
    return dotdot;
  }
