//======================================================================
//	mmaker.h
//	Version 2.00	date: 18 Mar 2003 08:58:50
//======================================================================
#ifndef MMAKER_H
#define MMAKER_H

#include <string>
using std::string;

#include <fstream>
using std::ifstream;
using std::ofstream;

#include <memory>		// Tom Hilinski, Dec2002 for auto_ptr

#include <v/vutil.h>        //vGetLocalTime
#include <v/vos.h>		// V OS Dependent Utils
#include <v/vslist.h>

class videCmdWindow;

  class makefileMaker
  {
    public:   //----------------------------------------------public

     makefileMaker();
     ~makefileMaker()
       {
       }

     void setDefaults();
     void eraseAll();

     //--- functions: makefile

     void saveMakefile ();

     //--- functions: V project

     void saveProject (char const * const n);
     bool loadProject (char const * const n);
     bool getSectionEntry (ifstream &prjf,
			const char* head,
			std::string & value);
     bool getSectionList (ifstream &prjf,
			const char* head,
			const char* term,
			vSList* sl);
     bool addFile (const char *n, bool addRelPath = true);
     bool addLibFile (const char *n);
     bool addInclude (const char *n);

     string getLibFiles();

     //--- functions: Set/Get member data

     void SetExeName (char const * const newText)
     	{
     	  exeName = newText;
     	  changed = true;
     	}
     string const & GetExeName () const
	{ return exeName; }
     void SetCC (char const * const newText)
     	{
     	  cc = newText;
     	  changed = true;
     	}
     string const & GetCC () const
	{ return cc; }
     void SetCFlags (char const * const newText)
     	{
     	  cFlags = newText;
     	  changed = true;
     	}
     string const & GetCFlags () const
	{ return cFlags; }

     void setLdFlags (char const * const newText)
     	{
     	  ldFlags = newText;
     	  changed = true;
     	}
     string const & getLdFlags () const
	{ return ldFlags; }

     void SetHomeV (char const * const newText)
     	{
     	  homeV = newText;
     	  changed = true;
     	}
     string const & GetHomeV () const
	{ return homeV; }
     void SetObjDir (char const * const newText)
     	{
     	  ObjDir = newText;
     	  changed = true;
     	}
     string const & GetObjDir () const
	{ return ObjDir; }
     void SetSrcDir (char const * const newText)
     	{
     	  SrcDir = newText;
     	  changed = true;
     	}
     string const & GetSrcDir () const
	{ return SrcDir; }
     void SetBinDir (char const * const newText)
     	{
     	  BinDir = newText;
     	  changed = true;
     	}
     string const & GetBinDir () const
	{ return BinDir; }
     void SetCtags (char const * const newText)
     	{
     	  ctags = newText;
     	  changed = true;
     	}
     string const & GetCtags () const
	{ return ctags; }
     void SetDebugSwitches (char const * const newText)
     	{
     	  debugSwitches = newText;
     	  changed = true;
     	}
     string const & GetDebugSwitches () const
	{ return debugSwitches; }
     void SetMakefileName (char const * const newText)
     	{
     	  makeName = newText;
     	  changed = true;
     	}
     string const & GetMakefileName () const
	{ return makeName; }

     //--- functions: Queries

     bool IsChanged () const
       { return changed; }

     // lists we need
     // changed to auto_ptrs - Tom Hilinski, Dec2002
     std::auto_ptr<vSList> projOpts;
     std::auto_ptr<vSList> incDirs;
     std::auto_ptr<vSList> libDirs;
     std::auto_ptr<vSList> srcFiles;
     std::auto_ptr<vSList> defsPool;
     std::auto_ptr<vSList> curDefs;
     std::auto_ptr<vSList> curOpts;		// User defines
     std::auto_ptr<vSList> rules;
     std::auto_ptr<vSList> userTargets;
     std::auto_ptr<vSList> libFiles;

     bool changed;	// true if member data has been modifed

     protected: //----------------------------------------------protected

     void fixSeparators(string & fullname);
     string getRelativePath(string & dot, string & file2);

     private:   //----------------------------------------------private

     void fixTabs (char* fixed, char* orig);
     const char* stripDir (const char* n);
     videCmdWindow* myCmdw;
     const char* srcN2ObjN (const char *, const char*);
     void findRule (const char * name, const char* comp);
     bool needCFile(const char* cF);
     const char* catSrc (const char* src, const char *file);

     // Changed the following from "char*" to std::string
     // to remove array overflow problems.
     // Note: these should be made private or protected, and have
     // overloaded interface functions return them as string or char*.
     // Tom Hilinski, Dec2002
     string BinDir;		// binary output directory
     string cc;			// compiler
     string cFlags;		// compiler flags
     string ctags;			// ctag flags
     string debugSwitches;		// debugger flags
     string exeName;		// output executable file name
     string homeV;			// V library source home directory
     // To Do: following should be a list
     string ldFlags;			// ld flags
     string makeName;		// makefile name
     string ObjDir;		// object file directory
     string SrcDir;		// source file directory

     string libOptsBCC;		// Borland compiler library options
     vSList libsBCC;			// list of Borland compiler libraries

  };

#endif
