//=======================================================================

//	videlib.cpp:	Source for videlib
//  Copyright (C) 1995-2000  Bruce E. Wampler
//	Version 2.00	date: 18 Mar 2003 10:26:08
//
//  This program is part of the V IDE
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  (see COPYING) along with this program; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=======================================================================

#include "videlib.h"
#include <v/vos.h>
#include <cstring>
#include <cctype>
    using namespace std;

//============================>>> stricmpV <<<===========================
  int stricmpV (char const * s1, char const * s2)
  {
    while (*s1 && std::toupper(*s1) == std::toupper(*s2))
      {
	s1++;
	s2++;
      }

    if ( *s1 == '\0' && *s2 == '\0' )
	return 0;
    else if ( *s1 == '\0' )
	return -1;
    else if ( *s2 == '\0' )
	return 1;
    else
	return ( std::toupper(*s1) - std::toupper(*s2) );
  }

//============================>>> splitFileName <<<===========================
  void splitFileName(const char* fn, char* fdir, char* fbase, char* fext)
  {
    // splits a file into three parts - the directory, the base name
    // (which includes the name+ext), and the extension (with .)
    // assumes output char*'s are at least as long as fn in.
    // Can be called with outputs NULL to skip

    int len = strlen(fn);	// length
    int ix;

    if (fdir)			// want dir?
      {
	strcpy(fdir,fn);		// copy it all
	for (ix = len ; ix > 0 ; --ix)
	  {
#ifdef V_VersionWindows
	    if ( IsPathSeparator (fn[ix]) || fn[ix] == ':')
#else
	    if ( IsPathSeparator (fn[ix]) )
#endif
	      {
		fdir[ix+1] = 0;
		break;
	      }
	  }
      }

    if (fbase)			// want base?
      {
	strcpy(fbase,fn);	// if no path
	for (ix = len ; ix > 0 ; --ix)
	  {
#ifdef V_VersionWindows
	    if ( IsPathSeparator (fn[ix]) || fn[ix] == ':')
#else
	    if ( IsPathSeparator (fn[ix]) )
#endif
	      {
		strcpy(fbase,&fn[ix+1]);
		break;
	      }
	  }
      }

    if (fext)			// want ext?
      {
	for (ix = len ; ix > 0 ; --ix)
	  {
	    if (fn[ix] == '.')
	      {
		strcpy(fext,&fn[ix]);
		break;
	      }
	  }
      }

  }

//============================>>> fixSlash <<<===========================
  void fixSlash(char* fullname)
  {
    // change \ to /
    while (*fullname)
      {
      	ToUnixPathSeparator ( fullname );
	++fullname;
      }
  }

//============================>>> fixSlash <<<===========================
  void fixSlash (std::string & fullname)
  {
    // change \ to /
    std::string::size_type i;
    while ( (i = fullname.find(DOS_PATH_SEPARATOR)) != std::string::npos)
	fullname[i] = UNIX_PATH_SEPARATOR;
  }

//============================>>> fixBackSlash <<<===========================
  const char* fixBackSlash(const char* fullname, int doit)
  {
    // change / to backslash
    string fn = fullname;
    return fixBackSlash(fn, doit).c_str();
  }

//============================>>> fixBackSlash <<<===========================
  std::string & fixBackSlash(std::string & fullname, int doit)
  {
    // change / to backslash
    if (doit)			// depends on OS
      {
	std::string::size_type i;
	while ( (i = fullname.find(UNIX_PATH_SEPARATOR)) != std::string::npos)
	    fullname[i] = DOS_PATH_SEPARATOR;
      }
    return fullname;
  }

//============================>>> fileQuote <<<===========================
  const char* fileQuote(const char* fname)
  {
    // adds quotes to file name on windows
    static char temp[maxFileNameSize + 2];
#ifdef V_VersionWindows
    if (strstr(fname," ") != 0)
      {
	strcpy(temp,"\"");
	strcat(temp,fname);
	strcat(temp,"\"");
	return temp;
      }
    else
	return fname;
#else
    return fname;
#endif
  }

//============================>>> fileQuote <<<===========================
  std::string & fileQuote(std::string & fname)
  {
    // adds quotes to file name on windows
#ifdef V_VersionWindows
    char const * const blankStr = " ";
    if ( fname.find (blankStr) != std::string::npos )	// have blank?
    {
    	char const * const dblQuote = "\"";
	fname.insert (0, dblQuote);
	fname += dblQuote;
    }
#else
#endif
    return fname;
  }

//============================>>> changeDir <<<===========================
  void changeDir(const char* fullname)
  {
    char temp[maxFileNameSize + 2];
    vOS vos;

    splitFileName(fullname , temp, 0, 0);	// get the path

    int len = strlen(temp);
    if (len < 1)
	return;

    if ( IsPathSeparator ( temp[len-1] ) )
	temp[--len] = NULL_CHAR;

#ifdef V_VersionWindows
    if (temp[1] == ':')		// a drive
      {
	int drive = temp[0]; // need to change drive, too.
	if (drive >= 'a')
	    drive = drive - ' ';
	vos.vChDrive(drive-'A');
      }
#endif
    vos.vChDir(temp);			// change dir
  }

//============================>>> Dos2UxFName <<<===========================
  void Dos2UxFName(
	char const * const dosFileName,
	char * const unixFileName,
	bool trimfile)
  {
    //makes things working with gnu-win32/ming32
    //translates a MSDOS file name into a unix like file name

    char* np = unixFileName;
    char const * p = dosFileName;

    for( ; *p ; p++, np++)
      {
	*np = (*p == DOS_PATH_SEPARATOR) ? UNIX_PATH_SEPARATOR : *p;
      }
    *np = NULL_CHAR;

    if (trimfile)
      {
	int const lim = strlen(unixFileName);
      	np = unixFileName + lim - 1;
	for (int ix = lim ; ix >= 0 ; --ix, --np)
	  {
	    if ( IsPathSeparator ( *np ) )
	      {
		*np = NULL_CHAR;	// zap rest
		break;
	      }
	  }
      }
  }

//===========================>>> getPath <<<===========================
  string getPath(string & fn)
  {
    string path = "." PATH_SEPARATOR_STR;
    string::size_type ix;
    for (ix = fn.length() ; ix > 0 ; --ix)
      {
#ifdef V_VersionWindows
	if ( IsPathSeparator (fn[ix]) || fn[ix] == ':')
#else
	if ( IsPathSeparator (fn[ix]) )
#endif
	  {
	    path = fn.substr(0,ix);	// not trailing /
	    break;
	  }
      }
    return path;
  }

//===========================>>> getFilePart <<<===========================
  string getFilePart(string & fn)
  {
    string path = fn;
    string::size_type ix;

    for (ix = fn.length() ; ix > 0 ; --ix)
      {
#ifdef V_VersionWindows
	if ( IsPathSeparator (fn[ix]) || fn[ix] == ':')
#else
	if ( IsPathSeparator (fn[ix]) )
#endif
	  {
	    path = fn.substr(ix+1,fn.length());
	    break;
	  }
      }
    return path;
  }

//===========================>>> ShortenFileName <<<===========================
// Provide a shortened file name representation, returning a string which is
// at most maxStringLength in length.
// Does not account for escaped chars e.g, '\\', so Windows paths will display
// less than maxStringLength chars.
// Tom Hilinski, Dec2002

  std::string ShortenFileName (char const * const fullFileName,
		unsigned short const maxStringLength)
  {
    std::string shortName = fullFileName;
    if (shortName.length() > maxStringLength)
    {
	char namePart[257], pathPart[257];
	splitFileName (fullFileName, pathPart, namePart, 0);
	shortName = namePart;
	short const lengthAvailable =
		maxStringLength - shortName.length() - 4;
	if (lengthAvailable > 10)
	{
	    std::string path = pathPart;
	    short const extractLength = lengthAvailable - 4;
	    shortName.insert (0,
		path.substr ( path.length() - extractLength, extractLength) );
	    shortName.insert (0, "....");
	}
    }
    return shortName;	// return contents on the stack
  }

//--- end of file ---
