//=======================================================================
//	videlib.h:	Source for videlib
//	Version 2.00	date: 17 Mar 2003 16:46:46
//  Copyright (C) 1995-2000  Bruce E. Wampler
//
//  This program is part of the V IDE
//=======================================================================
#ifndef VIDELIB_H
#define VIDELIB_H

#include <v/v_defs.h>
#include <string>
    using std::string;

#define NULL_CHAR		'\0'
#define DOS_PATH_SEPARATOR	'\\'
#define DOS_PATH_SEPARATOR_STR	"\\"
#define UNIX_PATH_SEPARATOR	'/'
#define UNIX_PATH_SEPARATOR_STR	"/"
#ifdef V_VersionX
  // *nix
  #define PATH_SEPARATOR	UNIX_PATH_SEPARATOR
  #define PATH_SEPARATOR_STR	UNIX_PATH_SEPARATOR_STR
  #define SRC_FILE_FILTER	"*.c;*.cpp;*.cc;*.cxx;*.C"
  #define HEADER_FILE_FILTER	"*.h;*.hpp;*.hh;*.hxx;*.H"
  #define FORTRAN_FILE_FILTER	"*.for;*.f;*.f77;*.f90;*.FOR;*.F;*.inc;*.INC"
#else
  // MS Windows
  #define PATH_SEPARATOR	DOS_PATH_SEPARATOR
  #define PATH_SEPARATOR_STR	DOS_PATH_SEPARATOR_STR
  #define SRC_FILE_FILTER	"*.c;*.cpp;*.cc;*.cxx"
  #define HEADER_FILE_FILTER	"*.h;*.hpp;*.hh;*.hxx"
  #define FORTRAN_FILE_FILTER	"*.for;*.f;*.f77;*.f90;*.inc"
#endif

// ---- library string constants ----

#define V_LIB_OPTION		"libV"
#define VX_LIB_OPTION		"libVx"
#define VGL_LIB_OPTION		"libVgl"
#define VXGL_LIB_OPTION		"libVxgl"
#define XATHENA_LIB_OPTION	" -lXaw -lXmu -lXt -lXext -lX11"
#define XATHENA_DEFINE		"-DAthena"
#define XMOTIF_LIB_OPTION	" -lXm -lXp -lXt -lXext -lX11"
#ifdef V_VersionX
  #define X_LIBRARY_PATH	"/usr/X11R6/lib"
  #define X_INCLUDE_PATH	"/usr/X11R6/include"
  #ifndef UseVGLw
    #define OGL_LIB_OPTION	" -lGLw -lGLU -lGL"
  #else
    #define OGL_LIB_OPTION	" -lGLU -lGL"
  #endif
#endif
#ifdef V_VersionWindows
  #define VDLL_OPTION		"libVDLL"
  #define VDLL_DEFINE		"-DUSE_V_DLL"
  #define GCC_NO_CYGWIN_OPTION	" -mno-cygwin"
  #define GCC_WINDOWS_OPTION	" -mwindows"
  #define OGL_LIB_OPTION	" -lglu32 -lopengl32"
#endif

// ---- Borland-related string constants ----

#define BORLAND_COMPILER	"BCC32"
#define BORLAND_DEBUG_FLAGS	"-v"
#define BORLAND_OPTIMIZE_FLAGS	"-O1 -v-"
#define BORLAND_STARTUP_GUI	"c0w32.obj"
#define BORLAND_STARTUP_CONSOLE	"c0x32.obj"
#define BORLAND_DLL_DEFINE	"-D_RTLDLL"

// ---- global functions ----

  int stricmpV (char const * s1, char const * s2);

  void fixSlash (char* fullname);
  void fixSlash (std::string & fullname);
  const char* fixBackSlash (const char* fn, int doit = 1);
  std::string & fixBackSlash (std::string & fn, int doit = 1);
  void splitFileName (const char* fn, char* fdir, char* fbase, char* fext);
  void changeDir (const char* fullname);
  void Dos2UxFName(char const * const dosFileName,
		   char * const unixFileName,
		   bool trimfile);
  const char* fileQuote (const char* fname);
  std::string & fileQuote (std::string & fname);

  inline
  std::string & normalizePath (		// nomalize a path to the host OS
    std::string & path)
    {
#ifdef V_VersionWindows
	return fixBackSlash (path,1);
#else
	return path;
#endif
    }

  inline
  const char* normalizePath (		// nomalize a path to the host OS
	const char* path)
    {
#ifdef V_VersionWindows
	return fixBackSlash (path,1);
#else
	return path;
#endif
    }

  string getRelativePath(const char* dot, const char* file2);
  string getPath(string & fn);
  string getFilePart(string & fn);

  std::string ShortenFileName (
	char const * const fullFileName,
	unsigned short const maxStringLength = 50);

  inline
  bool IsPathSeparator (		// is this a path delimiter char?
	char aChar)
    {
	return (aChar == DOS_PATH_SEPARATOR ||
		aChar == UNIX_PATH_SEPARATOR);
    }

  inline
  void ToDosPathSeparator (
	char* const aChar)
    {
	if (*aChar == UNIX_PATH_SEPARATOR)
		*aChar = DOS_PATH_SEPARATOR;
    }

  inline
  void ToUnixPathSeparator (
	char* const aChar)
    {
	if (*aChar == DOS_PATH_SEPARATOR)
		*aChar = UNIX_PATH_SEPARATOR;
    }

#endif
