//=======================================================================
// V IDE Make Class - all the stuff needed to create
// makefiles, and run make.
//	Version 2.00	date: 13 Mar 2003 09:11:30
//  Copyright (C) 1995-2000  Bruce E. Wampler
//
//  This program is part of the V C++ GUI Framework example programs.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  (see COPYING) along with this program; if not, write to the Free
//  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=======================================================================

#include <v/vfilesel.h>
#include <v/vnotice.h>

#include "videapp.h"

#include "videcmdw.h"

#include "videmake.h"
#include "videlib.h"

    static int filterIndex = 0;
    static char* filter[] =
      {
        "Make*;make*;Make*.*;make*.*",
        "*",
        0
      };

    char const * const videMake::errorOutputPrefix    = "  ! ";
    char const videMake::normalOutputPrefix = '>';
    const int videMake::makeOutMax = 2000;

//=====================>>> videMake::videMake <<<===========================
  videMake::videMake()
  {
    _stop = 0;
    cmdw = ((videApp*)theApp)->GetMsgWindow();

    makeTimer = new makeAuxTimer(this);	// create timer
    makeProc = new vPipedProcess();	// create process object
    makeOutLine.append (1024, NULL_CHAR);
  }

//=====================>>> videMake::~videMake <<<===========================
  videMake::~videMake()
  {
    makeTimer->TimerStop();
    delete makeTimer;
    makeProc->killProcess();
    delete makeProc;
  }

//=====================>>> videMake::DoMake <<<===========================
  int videMake::DoMake(videCmdWindow* parent, char* what)
  {
    static char makebName[maxFileNameSize+2] = "make";
    // setup make command - handles name of make tool to work around
    // the MinGW mingw32-make.exe problem. VIDE version 1.26
    // defaults for Linux and Windows first

    // @@@@ To Fix: makebName to a string

    strcpy(makebName,(const char*)((videApp*)theApp)->getMakeProg());
    if (!*makebName)
	strcpy(makebName,"make");
#ifdef V_VersionWindows
    if (((videApp*)theApp)->getCompiler() == bcc)
      {
	strcpy(makebName,((videApp*)theApp)->getCCPath());
	if (*makebName)
	    strcat(makebName,"\\bin\\make");
	else
	    strcpy(makebName,"make");
      }
#endif
#ifdef V_VersionWindows
    // this call to sleep is needed because 
    if (((videApp*)theApp)->getCompiler() == MinGW)
      {
	::Sleep((DWORD) 600);
      }
#endif

    int retv = DoMake0( parent, (char *)makebName, what);
    if (retv == -1)
      {
        if (strcmp(makebName,"make") != 0)
	  {
	    cmdw->AddLine("Unable to make, trying alternate name for make tool.");
	     strcpy(makebName,"make");	// try with just make
	     retv = DoMake0( parent, (char *)makebName, what);
	     if (retv == 1)		// worked this time
		{
		  cmdw->AddLine("Make program automatically changed to \"make\".");
		  ((videApp*)theApp)->setMakeProg("make");  // change silently
		  return retv;
		}
	  }
	else if (((videApp*)theApp)->getCompiler() == MinGW)
	  {
	    cmdw->AddLine("Unable to make, trying alternate name for make tool.");
	     strcpy(makebName,"mingw32-make");	// try with just make
	     retv = DoMake0( parent, (char *)makebName, what);
	     if (retv == 1)		// worked this time
		{
		  cmdw->AddLine("Make program automatically changed to \"mingw32-make\".");
		  ((videApp*)theApp)->setMakeProg("mingw32-make");  // change silently
		  return retv;
		}
	  }

	cmdw->AddLine("Unable to create make process, can't find \"make\" tool.");
	return retv;
      }
    else
	return retv;
  }

//=====================>>> videMake::DoMake0 <<<===========================
  int videMake::DoMake0(
  	videCmdWindow* parent,
  	char const * const makecmd,
  	char const * const what)
  {
    // New version of DoMake for V 1.11, 1.12 - uses new vPipedProcess class
    // changed cmd from char[] to std::string - Tom Hilinski, Dec2002
    // char cmd[maxFileNameSize];
    std::string cmd;
    vOS vos;
    int isBorland = ((videApp*)theApp)->getCompiler() == bcc;

    char* mkpath = ((videApp*)theApp)->GetMkFile();
    cmdw = ((videApp*)theApp)->GetMsgWindow();

    if (!*mkpath)
      {
        vNoticeDialog note((vWindow*)cmdw);
        note.Notice("No Makefile specified. Use Select Makefile or Open Project first.");
        return 0;
      }

    if (isBorland)
	(void)fixBackSlash(mkpath,isBorland);
    else
	fixSlash(mkpath);

    cmdw->RaiseWindow();
    cmdw->SetRdOnly(0);
    if (isBorland)
	(cmdw->GetTextEd())->SetFileType(bccError);
    else
	(cmdw->GetTextEd())->SetFileType(gccError);

    // Now, change dir

    changeDir(mkpath);

    // build command again without what part
    cmd = makecmd;

    if (isBorland)
	cmd += " -f";			// -f for file (no space after "f")
    else
	cmd += " -f ";			// -n for dry run, -f for file
#ifdef V_VersionWindows
    if (strstr(mkpath," ") != 0)	// has space in name
      {
      	cmd += '\"';			// add quotes for space
      	cmd += mkpath;			// the name of the make file.
      	cmd += '\"';
      }
    else
#endif
    cmd += mkpath;			// the name of the make file.

    if (what && *what)
      {
      	cmd += ' ';
      	cmd += what;
      }
    cmdw->AddLine( cmd.c_str() );

    theApp->CheckEvents();
    _stop = 0;			// not stopping

    makeOutLine = errorOutputPrefix;			// set up output line

    // ok, have everything ready to go in cmd...

    if (!makeProc->startPipedProcess( cmd.c_str() ))
      {
	cmdw->AddLine("Unable to create make process....");
	cmdw->AddLine( cmd.c_str() );
	return 0;
      }

    // Now, start the timer to monitor make progress

    makeTimer->TimerSet(300);		// update 3 times a second

    // make is now running and handled by its timer

    return 1;
  }

//==================>>> videMake::checkMakeProgress <<<======================
  void videMake::checkMakeProgress()
  {
    // this will try to echo the output of the Make, and terminate the
    // make on _stop or when done.
    int bytesRead;

    char chrin[10] = "";

    // First, read any characters output
    for ( ; ; )		// read all debugger output available
      {
	// first, see if any output is available...
	bytesRead = makeProc->readChar(chrin);
	if (bytesRead == 0)		// nothing to read!
	    break;
	if (*chrin == '\r')		// ignore these
	     continue;
	else if (*chrin == '\n')		// end of line
	  {
	    if ( makeOutLine.find (": ") != std::string::npos )	// error line
	      {
	      }
	    else						// normal line
	      {
		makeOutLine[2] = normalOutputPrefix;
		makeOutLine.assign (
			makeOutLine, 2, std::string::npos );
	      }
	    cmdw->AddLine( makeOutLine.c_str() );
	    // start a new command
	    makeOutLine = errorOutputPrefix;
	  }
	else
	  {
	    makeOutLine += *chrin;
	    if (makeOutLine.length() >= makeOutMax)
	      {
		if ( makeOutLine.find (": ") != std::string::npos ) // error line
		  {
		  }
		else						// normal line
		  {
		    makeOutLine[2] = normalOutputPrefix;
		    makeOutLine.assign (
			makeOutLine, 2, std::string::npos );
		  }
		cmdw->AddLine( makeOutLine.c_str() );
		// start a new command
		makeOutLine = errorOutputPrefix;
	      }
	  }
      }

    // Second, check for stop
    theApp->CheckEvents();
    if (_stop)
      {
	_stop = 0;
	makeTimer->TimerStop();
	cmdw->AddLine("---- Make Aborted ----");
	cmdw->SetRdOnly(1);
	cmdw->RaiseWindow();
	theApp->CheckEvents();
	makeProc->killProcess();
	return;
      }

    // Third, see if make is finished
    if ( !makeProc->stillRunning() )	// signaled, means done
      {
	makeTimer->TimerStop();
	if (makeProc->getExitCode() != 0)
	    cmdw->AddLine("---- Make had errors. Right click error line to open file.");
	else
	    cmdw->AddLine("---- Make finished ----");
	cmdw->SetRdOnly(1);
	cmdw->RaiseWindow();
	theApp->CheckEvents();
      }

  }

//==================>>> videMake::TimeTick <<<======================
  void makeAuxTimer::TimerTick()
  {
    makeIt->checkMakeProgress();
  }

