//===============================================================
// vPrefs Class for working with .ini or .rc files
//	Version 2.00	date: 14 Mar 2003 11:32:34
// Copyright (C) 1995-2003  Bruce E. Wampler
//===============================================================

#include <v/v_defs.h>

#include "vprefs.h"
#include "videlib.h"
#include <v/vutil.h>

#include <v/vos.h>

//===========================>>> vPrefs::vPrefs <<<=======================
  vPrefs::vPrefs()
  {
  }

//===========================>>> vPrefs::~vPrefs <<<=======================
  vPrefs::~vPrefs()
  {
  }

//===========================>>> vPrefs::openSave <<<=======================
  int vPrefs::openSave(const char* name, const char* title, const char* subdir)
  {
    // called with a base name of the file and a title line...
    char winDir[maxFileNameSize+20];

#ifdef V_VersionWindows
    vOS os;
    if (os.vGetEnvVal("APPDATA", winDir, maxFileNameSize))
      {
	if (subdir && *subdir)
	  {
	    strcat(winDir, PATH_SEPARATOR_STR);
	    strcat(winDir, subdir);
	    ::CreateDirectory(winDir,0);		// be sure directory is there
	  }
      }
    else if (!::GetWindowsDirectory(winDir, maxFileNameSize))
	return 0;

    strcat(winDir, PATH_SEPARATOR_STR);
    strcat(winDir, name);
    strcat(winDir, ".ini");
#endif

#ifdef V_VersionX
    vOS os;

    if (!os.vGetEnvVal("HOME", winDir, maxFileNameSize))
      {
	strcpy(winDir, "." PATH_SEPARATOR_STR);
	strcat(winDir, name);
	strcat(winDir, "rc");
      }
    else
      {
	strcat(winDir, PATH_SEPARATOR_STR ".");
	strcat(winDir, name);
	strcat(winDir, "rc");
      }
#endif

    outFile.open(winDir);
    if (!outFile)
	return 0;
    outFile << title << std::endl;
    return 1;

  }

//===========================>>> vPrefs::closeSave <<<=======================
  void vPrefs::closeSave()
  {
    outFile.close();
  }

//===========================>>> vPrefs::openRead <<<=======================
  int vPrefs::openRead(const char* name, const char* title, const char* subdir)
  {
    char winDir[maxFileNameSize+20];


#ifdef V_VersionWindows
    vOS os;
    if (os.vGetEnvVal("APPDATA", winDir, maxFileNameSize))
      {
	if (subdir && *subdir)
	  {
	    strcat(winDir, PATH_SEPARATOR_STR);
	    strcat(winDir, subdir);
	  }
      }
    else if (!::GetWindowsDirectory(winDir, maxFileNameSize))
	    return 0;
    strcat(winDir, PATH_SEPARATOR_STR);
    strcat(winDir, name);
    strcat(winDir, ".ini");
#endif
#ifdef V_VersionX
    vOS os;
    if (!os.vGetEnvVal("HOME", winDir, maxFileNameSize))
      {
	strcpy(winDir, "." PATH_SEPARATOR_STR);
	strcat(winDir, name);
	strcat(winDir, "rc");
      }
    else
      {
	strcat(winDir, PATH_SEPARATOR_STR ".");
	strcat(winDir, name);
	strcat(winDir, "rc");
      }
#endif

    inFile.open(winDir);
    if (!inFile)
	return 0;
    return 1;

  }

//===========================>>> vPrefs::closeRead <<<=======================
  void vPrefs::closeRead()
  {
    inFile.close();
  }

//===========================>>> vPrefs::getString <<<=======================
  int vPrefs::getString(const char* entry, char* str)
  {
    char buff[256];
    char name[64];
    char* np;
    char* bp;

    inFile.seekg(0, std::ios::beg);
    if (!inFile)
	return 0;
    for ( ; ; )
      {
	inFile.getline(buff,255);
	if (!inFile)
	{
	    inFile.clear();
	    break;
	}
        if (*buff == '[')	// skip section lines
            continue;

 	np = name;
	for (bp = buff ; *bp && *bp != '=' ; )
	    *np++ = *bp++;
	*np = 0;
	if (*bp != '=')
	    return 0;
	bp++;
	if (strcmp(name,entry) != 0)
	    continue;
	strcpy(str,bp);
	return 1;
      }

    return 0;
  }

//===========================>>> vPrefs::getLong <<<=======================
  int vPrefs::getLong(const char* entry, long& val)
  {
    char buff[256];
    char name[64];
    char* np;
    char* bp;


    inFile.seekg(0, std::ios::beg);
    if (!inFile)
	return 0;
    for ( ; ; )
      {
	inFile.getline(buff,255);
	if (!inFile)
	{
	    inFile.clear();
	    break;
	}
        if (*buff == '[')	// skip section lines
            continue;

 	np = name;
	for (bp = buff ; *bp && *bp != '=' ; )
	    *np++ = *bp++;
	*np = 0;
	if (*bp != '=')
	    return 0;
	bp++;
	if (strcmp(name,entry) != 0)
	    continue;
	strcpy(name,bp);
	val = StrToLong(name);
	return 1;
      }

    return 0;
  }

//===========================>>> vPrefs::addString <<<=======================
  void vPrefs::addString(const char* entry, const char* str)
  {
    outFile << entry << "=" << str << std::endl;
  }

//===========================>>> vPrefs::addLong <<<=======================
  void vPrefs::addLong(const char* entry, long val)
  {
    outFile << entry << "=" << val << std::endl;
  }
